@@ -7,6 +7,8 @@ This package provides several entry points
        * compile_cxxcode: c++ (str) to DLL, returns DLL filename
        * compile_pythrancode: python (str) to so/cpp, returns output filename
        * compile_pythranfile: python (file) to so/cpp, returns output filename
+       * import_pythrancode: python (str) to module, returns loaded module
+       * import_pythranfile: python (file) to module, returns loaded module
        * test_compile: passthrough compile test, raises CompileError Exception.
 
 Basic scenario is to turn a Python AST into C++ code:
@@ -30,6 +32,12 @@ Higher level entry points include:
 >>> dll_file = compile_pythrancode("my_module", export + code)
 >>> dll_file = compile_cxxfile("my_module", cpp_file)
 
+It is possible to directly turn pythran code into an imported module:
+>>> code = '#pythran export greet()\\ndef greet(): return "demat"'
+>>> greeter = import_pythrancode(code)
+>>> greeter.greet()
+'demat'
+
 Cleanup
 >>> import os, glob
 >>> for target in glob.glob('my_module.*'):
@@ -41,6 +49,7 @@ import pythran.log
 from pythran.config import get_include
 from pythran.toolchain import (generate_cxx, compile_cxxfile, compile_cxxcode,
                                compile_pythrancode, compile_pythranfile,
+                               import_pythrancode, import_pythranfile,
                                test_compile)
 from pythran.spec import spec_parser
 from pythran.spec import load_specfile
