@@ -10,6 +10,13 @@ PYTHONIC_NS_BEGIN
 
 namespace numpy
 {
+  template <class dtype>
+  typename dtype::type
+  empty(types::pshape<> const &shape, dtype)
+  {
+    return {};
+  }
+
   template <class pS, class dtype>
   types::ndarray<typename dtype::type, sutils::shape_t<pS>>
   empty(pS const &shape, dtype)
