@@ -10,6 +10,14 @@ PYTHONIC_NS_BEGIN
 
 namespace numpy
 {
+
+  template <class dtype>
+  typename dtype::type
+  zeros(std::tuple<> const &shape, dtype d)
+  {
+    return static_cast<typename dtype::type>(0);
+  }
+
   template <class pS, class dtype>
   types::ndarray<typename dtype::type, sutils::shape_t<pS>>
   zeros(pS const &shape, dtype d)
