@@ -25,10 +25,15 @@ import pythran.frontend as frontend
 from datetime import datetime
 from distutils.errors import CompileError
 from distutils import sysconfig
-from numpy.distutils.core import setup
+try:
+    # `numpy.distutils is deprecated, may not be present, or broken
+    from numpy.distutils.core import setup
+except Exception:
+    from distutils.core import setup
 
 from tempfile import mkdtemp, NamedTemporaryFile
 import gast as ast
+import importlib
 import logging
 import os.path
 import shutil
@@ -87,10 +92,10 @@ def has_argument(module, fname):
 
 
 def front_middle_end(module_name, code, optimizations=None, module_dir=None,
-                     entry_points=None):
+                     entry_points=None, report_times = False):
     """Front-end and middle-end compilation steps"""
 
-    pm = PassManager(module_name, module_dir)
+    pm = PassManager(module_name, module_dir, code)
 
     # front end
     ir, docstrings = frontend.parse(pm, code)
@@ -102,7 +107,7 @@ def front_middle_end(module_name, code, optimizations=None, module_dir=None,
     if optimizations is None:
         optimizations = cfg.get('pythran', 'optimizations').split()
     optimizations = [_parse_optimization(opt) for opt in optimizations]
-    refine(pm, ir, optimizations)
+    refine(pm, ir, optimizations, report_times)
 
     return pm, ir, docstrings
 
@@ -110,19 +115,19 @@ def front_middle_end(module_name, code, optimizations=None, module_dir=None,
 # PUBLIC INTERFACE STARTS HERE
 
 
-def generate_py(module_name, code, optimizations=None, module_dir=None):
+def generate_py(module_name, code, optimizations=None, module_dir=None, report_times=False):
     '''python + pythran spec -> py code
 
     Prints and returns the optimized python code.
 
     '''
 
-    pm, ir, _ = front_middle_end(module_name, code, optimizations, module_dir)
+    pm, ir, _ = front_middle_end(module_name, code, optimizations, module_dir, report_times=report_times)
     return pm.dump(Python, ir)
 
 
 def generate_cxx(module_name, code, specs=None, optimizations=None,
-                 module_dir=None):
+                 module_dir=None, report_times=False):
     '''python + pythran spec -> c++ code
     returns a PythonModule object and an error checker
 
@@ -137,6 +142,7 @@ def generate_cxx(module_name, code, specs=None, optimizations=None,
 
     pm, ir, docstrings = front_middle_end(module_name, code, optimizations,
                                           module_dir,
+                                          report_times=report_times,
                                           entry_points=entry_points)
 
     # back-end
@@ -364,7 +370,7 @@ def compile_cxxcode(module_name, cxxcode, output_binary=None, keep_temp=False,
 
 def compile_pythrancode(module_name, pythrancode, specs=None,
                         opts=None, cpponly=False, pyonly=False,
-                        output_file=None, module_dir=None, **kwargs):
+                        output_file=None, module_dir=None, report_times=False, **kwargs):
     '''Pythran code (string) -> c++ code -> native module
 
     if `cpponly` is set to true, return the generated C++ filename
@@ -375,7 +381,7 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
 
     if pyonly:
         # Only generate the optimized python code
-        content = generate_py(module_name, pythrancode, opts, module_dir)
+        content = generate_py(module_name, pythrancode, opts, module_dir, report_times)
         if output_file is None:
             print(content)
             return None
@@ -392,7 +398,7 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
 
     # Generate C++, get a PythonModule object
     module, error_checker = generate_cxx(module_name, pythrancode, specs, opts,
-                                         module_dir)
+                                         module_dir, report_times)
 
     if 'ENABLE_PYTHON_MODULE' in kwargs.get('undef_macros', []):
         module.preamble.insert(0, Line('#undef ENABLE_PYTHON_MODULE'))
@@ -409,6 +415,9 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
         shutil.move(tmp_file, output_file)
         logger.info("Generated C++ source file: " + output_file)
     else:
+        if not specs:
+            raise ValueError("Empty spec files while generating native module")
+
         # Compile to binary
         try:
             output_file = compile_cxxcode(module_name,
@@ -419,14 +428,31 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
             logger.warning("Compilation error, "
                            "trying hard to find its origin...")
             error_checker()
-            logger.warning("Nop, I'm going to flood you with C++ errors!")
+            logger.warning("Nope, I'm going to flood you with C++ errors!")
             raise
 
     return output_file
 
 
+def import_pythrancode(pythrancode, **kwargs):
+    # It should be safe to delete the library once it's loaded in memory.
+    digest = hashlib.sha256(pythrancode.encode()).hexdigest()
+    module_name = "pythranized_{}".format(digest)
+    tmpfile = None
+    try:
+        tmpfile = compile_pythrancode(module_name, pythrancode, **kwargs)
+        spec = importlib.util.spec_from_file_location(module_name, tmpfile)
+        module = importlib.util.module_from_spec(spec)
+        sys.modules[module_name] = module
+        spec.loader.exec_module(module)
+        return module
+    finally:
+        if tmpfile is not None:
+            os.remove(tmpfile)
+
+
 def compile_pythranfile(file_path, output_file=None, module_name=None,
-                        cpponly=False, pyonly=False, **kwargs):
+                        cpponly=False, pyonly=False, report_times=False, **kwargs):
     """
     Pythran file -> c++ file -> native module.
 
@@ -447,7 +473,7 @@ def compile_pythranfile(file_path, output_file=None, module_name=None,
     Specify the output file:
 
     >>> import sysconfig
-    >>> ext = sysconfig.get_config_vars()["SO"]
+    >>> ext = sysconfig.get_config_vars()["EXT_SUFFIX"]
     >>> so_path = compile_pythranfile('pythran_test.py', output_file='foo'+ext)
     """
     if not output_file:
@@ -474,6 +500,7 @@ def compile_pythranfile(file_path, output_file=None, module_name=None,
                                               output_file=output_file,
                                               cpponly=cpponly, pyonly=pyonly,
                                               module_dir=module_dir,
+                                              report_times=report_times,
                                               **kwargs)
     except PythranSyntaxError as e:
         if e.filename is None:
@@ -483,6 +510,11 @@ def compile_pythranfile(file_path, output_file=None, module_name=None,
     return output_file
 
 
+def import_pythranfile(pythranpath, **kwargs):
+    with open(pythranpath) as fd:
+        return import_pythrancode(fd.read(), **kwargs)
+
+
 def test_compile():
     '''Simple passthrough compile test.
     May raises CompileError Exception.
